package blusunrize.immersiveengineering.common.gui;

import blusunrize.immersiveengineering.api.ApiUtils;
import blusunrize.immersiveengineering.api.crafting.ArcFurnaceRecipe;
import blusunrize.immersiveengineering.api.crafting.BlastFurnaceRecipe;
import blusunrize.immersiveengineering.api.crafting.BlueprintCraftingRecipe;
import blusunrize.immersiveengineering.api.shader.CapabilityShader;
import blusunrize.immersiveengineering.api.shader.CapabilityShader.ShaderWrapper;
import blusunrize.immersiveengineering.api.shader.IShaderItem;
import blusunrize.immersiveengineering.api.tool.IConfigurableTool;
import blusunrize.immersiveengineering.api.tool.IDrillHead;
import blusunrize.immersiveengineering.api.tool.IUpgrade;
import blusunrize.immersiveengineering.api.tool.IUpgradeableTool;
import blusunrize.immersiveengineering.common.IEContent;
import blusunrize.immersiveengineering.common.items.ItemBullet;
import blusunrize.immersiveengineering.common.items.ItemEngineersBlueprint;
import blusunrize.immersiveengineering.common.util.IEAchievements;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fluids.FluidUtil;
import net.minecraftforge.fluids.capability.IFluidHandler;
import net.minecraftforge.fluids.capability.IFluidTankProperties;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.OreDictionary;

public abstract class IESlot extends Slot
{
	final Container container;
	public IESlot(Container container, IInventory inv, int id, int x, int y)
	{
		super(inv, id, x, y);
		this.container=container;
	}

	@Override
	public boolean func_75214_a(ItemStack itemStack)
	{
		return true;
	}

	public static class Output extends IESlot
	{
		public Output(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return false;
		}
		@Override
		public void func_82870_a(EntityPlayer player, ItemStack stack)
		{
			super.func_82870_a(player, stack);
			if(player!=null && (this.container instanceof ContainerBlastFurnace || this.container instanceof ContainerArcFurnace) && ApiUtils.compareToOreName(stack, "ingotSteel"))
				player.func_71029_a(IEAchievements.makeSteel);
		}
	}
	public static class FluidContainer extends IESlot
	{
		boolean empty;
		public FluidContainer(Container container, IInventory inv, int id, int x, int y, boolean empty)
		{
			super(container, inv, id, x, y);
			this.empty=empty;
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			IFluidHandler handler = FluidUtil.getFluidHandler(itemStack);
			if(handler == null || handler.getTankProperties() == null)
				return false;
			IFluidTankProperties[] tank = handler.getTankProperties();
			if(tank == null || tank.length < 1 || tank[0] == null)
				return false;
			if(empty)
				return tank[0].getContents() == null;
			else
				return tank[0].getContents() != null;
		}
	}
	public static class BlastFuel extends IESlot
	{
		public BlastFuel(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return BlastFurnaceRecipe.isValidBlastFuel(itemStack);
		}
	}
	public static class Bullet extends IESlot
	{
		int limit;
		public Bullet(Container container, IInventory inv, int id, int x, int y, int limit)
		{
			super(container, inv, id, x, y);
			this.limit=limit;
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return itemStack != null && itemStack.func_77973_b() instanceof ItemBullet;
		}
		@Override
		public int func_75219_a()
		{
			return limit;
		}
	}
	public static class DrillHead extends IESlot
	{
		public DrillHead(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return itemStack!=null && itemStack.func_77973_b() instanceof IDrillHead;
		}
		@Override
		public int func_75219_a()
		{
			return 1;
		}
	}
	public static class Upgrades extends IESlot
	{
		ItemStack upgradeableTool;
		String type;
		boolean preventDoubles;
		public Upgrades(Container container, IInventory inv, int id, int x, int y, String type, ItemStack upgradeableTool, boolean preventDoubles)
		{
			super(container, inv, id, x, y);
			this.type = type;
			this.upgradeableTool = upgradeableTool;
			this.preventDoubles = preventDoubles;
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			if(preventDoubles)
				for(Slot slot : container.field_75151_b)
					if(slot instanceof Upgrades && ((Upgrades)slot).preventDoubles && OreDictionary.itemMatches(slot.func_75211_c(), itemStack, true))
						return false;
			return itemStack!=null && itemStack.func_77973_b() instanceof IUpgrade && ((IUpgrade)itemStack.func_77973_b()).getUpgradeTypes(itemStack).contains(type) && ((IUpgrade)itemStack.func_77973_b()).canApplyUpgrades(upgradeableTool, itemStack);
		}
		@Override
		public int func_75219_a()
		{
			return 64;
		}
	}
	public static class Shader extends IESlot
	{
		ItemStack tool;
		public Shader(Container container, IInventory inv, int id, int x, int y, ItemStack tool)
		{
			super(container, inv, id, x, y);
			this.tool = tool;
			if(FMLCommonHandler.instance().getEffectiveSide()==Side.CLIENT)
				this.setBackgroundName("immersiveengineering:items/shader_slot");
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			if(itemStack==null || !(itemStack.func_77973_b() instanceof IShaderItem) || tool==null || !tool.hasCapability(CapabilityShader.SHADER_CAPABILITY,null))
				return false;
			ShaderWrapper wrapper = tool.getCapability(CapabilityShader.SHADER_CAPABILITY,null);
			if(wrapper==null)
				return false;
			return ((IShaderItem)itemStack.func_77973_b()).getShaderCase(itemStack, tool, wrapper.getShaderType())!=null;
		}
		@Override
		public int func_75219_a()
		{
			return 1;
		}
	}
	public static class UpgradeableItem extends IESlot
	{
		int size;
		public UpgradeableItem(Container container, IInventory inv, int id, int x, int y, int size)
		{
			super(container, inv, id, x, y);
			this.size = size;
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			if(itemStack==null)
				return false;
			if(itemStack.func_77973_b() instanceof IUpgradeableTool)
				return ((IUpgradeableTool)itemStack.func_77973_b()).canModify(itemStack);
			if(itemStack.func_77973_b() instanceof IConfigurableTool)
				return ((IConfigurableTool)itemStack.func_77973_b()).canConfigure(itemStack);
			return false;
		}
		@Override
		public int func_75219_a()
		{
			return size;
		}
		@Override
		public void func_75218_e()
		{
			super.func_75218_e();
			if(container instanceof ContainerModWorkbench)
				((ContainerModWorkbench)container).rebindSlots();
		}
		@Override
		public boolean func_82869_a(EntityPlayer player)
		{
			return !(this.func_75211_c() != null && func_75211_c().func_77973_b() instanceof IUpgradeableTool && !((IUpgradeableTool) func_75211_c().func_77973_b()).canTakeFromWorkbench(func_75211_c()));
		}
		@Override
		public void func_82870_a(EntityPlayer player, ItemStack stack)
		{
			super.func_82870_a(player, stack);
			if(stack!=null && stack.func_77973_b() instanceof IUpgradeableTool)
				((IUpgradeableTool)stack.func_77973_b()).removeFromWorkbench(player, stack);

		}
	}
	public static class Ghost extends IESlot
	{
		public Ghost(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}

		@Override
		public void func_75215_d(ItemStack itemStack)
		{
			super.func_75215_d(itemStack);
		}
		@Override
		public boolean func_82869_a(EntityPlayer player)
		{
			return false;
		}
		@Override
		public int func_75219_a()
		{
			return 1;
		}
	}
	public static class ItemDisplay extends IESlot
	{
		public ItemDisplay(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return false;
		}
		@Override
		public boolean func_82869_a(EntityPlayer player)
		{
			return false;
		}
	}

	public static class BlueprintInput extends IESlot
	{
		ItemStack upgradeableTool;
		public BlueprintInput(Container container, IInventory inv, int id, int x, int y, ItemStack upgradeableTool)
		{
			super(container, inv, id, x, y);
			this.upgradeableTool = upgradeableTool;
		}
		@Override
		public void func_75218_e()
		{
			this.field_75224_c.func_70296_d();
			if(upgradeableTool!=null && upgradeableTool.func_77973_b() instanceof ItemEngineersBlueprint)
				((ItemEngineersBlueprint)upgradeableTool.func_77973_b()).updateOutputs(upgradeableTool);
			if(container instanceof ContainerModWorkbench)
				((ContainerModWorkbench)container).rebindSlots();
			super.func_75218_e();
		}
		@Override
		public int func_75219_a()
		{
			return 64;
		}
	}
	public static class BlueprintOutput extends IESlot
	{
		public BlueprintCraftingRecipe recipe;
		ItemStack upgradeableTool;
		public BlueprintOutput(Container container, IInventory inv, int id, int x, int y, ItemStack upgradeableTool, BlueprintCraftingRecipe recipe)
		{
			super(container, inv, id, x, y);
			this.upgradeableTool = upgradeableTool;
			this.recipe = recipe;
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return false;
		}


		@Override
		@SideOnly(Side.CLIENT)
		public boolean func_111238_b()
		{
			return this.func_75216_d();
		}

		@Override
		public void func_82870_a(EntityPlayer player, ItemStack stack)
		{
			if(upgradeableTool!=null && upgradeableTool.func_77973_b() instanceof ItemEngineersBlueprint)
				((ItemEngineersBlueprint)upgradeableTool.func_77973_b()).reduceInputs(recipe, upgradeableTool, stack, this.container);
			if(container instanceof ContainerModWorkbench)
			{
				((ContainerModWorkbench)container).rebindSlots();
				((ContainerModWorkbench)container).tile.func_70296_d();
			}
			for(IEAchievements.AchievementIE achievement : IEAchievements.blueprintCraftingAchievements)
			{
				if(achievement.triggerItems!=null && achievement.triggerItems.length>0)
				{
					for(ItemStack trigger : achievement.triggerItems)
						if(ApiUtils.stackMatchesObject(stack, trigger, achievement.checkNBT&&trigger.func_77942_o()))
						{
							player.func_71029_a(achievement);
							break;
						}
				} else if(ApiUtils.stackMatchesObject(stack, achievement.field_75990_d, achievement.checkNBT&&achievement.field_75990_d.func_77942_o()))
					player.func_71029_a(achievement);
			}
			this.field_75224_c.func_70296_d();
			super.func_82870_a(player, stack);
		}
	}

	public static class ArcInput extends IESlot
	{
		public ArcInput(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return itemStack!=null && ArcFurnaceRecipe.isValidRecipeInput(itemStack);
		}
	}
	public static class ArcAdditive extends IESlot
	{
		public ArcAdditive(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return itemStack!=null && ArcFurnaceRecipe.isValidRecipeAdditive(itemStack);
		}
	}
	public static class ArcElectrode extends IESlot
	{
		public ArcElectrode(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			return itemStack!=null && IEContent.itemGraphiteElectrode.equals(itemStack.func_77973_b());
		}
	}

	public static class ContainerCallback extends IESlot
	{
		public ContainerCallback(Container container, IInventory inv, int id, int x, int y)
		{
			super(container, inv, id, x, y);
		}
		@Override
		public boolean func_75214_a(ItemStack itemStack)
		{
			if(this.container instanceof ICallbackContainer)
				return ((ICallbackContainer)this.container).canInsert(itemStack, field_75222_d, this);
			return true;
		}
		@Override
		public boolean func_82869_a(EntityPlayer player)
		{
			if(this.container instanceof ICallbackContainer)
				return ((ICallbackContainer)this.container).canTake(this.func_75211_c(), field_75222_d, this);
			return true;
		}
	}
	public interface ICallbackContainer
	{
		boolean canInsert(ItemStack stack, int slotNumer, Slot slotObject);
		boolean canTake(ItemStack stack, int slotNumer, Slot slotObject);
	}
}
